
# in package main.

$| = 1;

use CGI qw(:standard);
use CGI::Carp qw(fatalsToBrowser);


my %script_info = (
                       # next state     title
  'jobupload.cgi' => [ 'jobcheck.cgi', 'Document Collection Upload' ],
  'jobcheck.cgi'  => [ 'jobcheck.cgi', 'Annotation Job Status' ],
);



$incoming_dir = 'incoming';
$outgoing_dir = 'outgoing';
$jobs_dir     = 'jobs';

#
# in actual fact, all operations on a Document Collection
# are performed within the incoming directory.
# the outgoing directory is only for making downloadable files available.
# the jobs directory contains the jobs database files.
#

unless ( $no_cgi ) {
  $query = new CGI;
  $script_name = $query->url(-relative=>1);
  $next_script_name = $script_info{$script_name}[0];
}


sub my_header {
  my $title = $script_info{$script_name}[1];

  (
    $query->header( -expires => '-1d' ),
    $query->start_html(
      -TITLE      => $title,
      -AUTHOR     => 'jporter@logicon.com',
      -BGCOLOR    => '#FFFFEE'
    ),
    $query->h2( $title )
  )
}


# arg is a job name.
# returns a cgi query object.
# dies with message on failure.
sub load_job {
  my $job_name = Id::validate_name( shift );
  $job_name or return undef;

  my $file = $jobs_dir . '/' . $job_name;

  open F, "< $file" or return undef;

  my $jobq = CGI->new(F);
  close F;
  $jobq;
}

# arg is a cgi query object.
# dies with message on failure.
sub save_job {
  my $jobq = shift;
  my $file = $jobs_dir . '/' . $jobq->param('job_name');

  open F, "> $file" or die "Error opening $file: $!\n";

  $jobq->save(F);
  close F;
  $jobq;
}


sub add_action_item {
  my( $jobq, $action_name, $status ) = @_;
  my $time = time;
  $jobq->append(
     -NAME => 'action',
   -VALUES => [ "$action_name - $status - $time" ]
  );
  $jobq;
}


sub job_actions {
  my $jobq = shift;
  map { [ split / - / ] } $jobq->param('action');
}


#
# arg is a cgi query object.
# returns an array of actions for which the most recent activity
# was 'Request'.
# order is from oldest to newest.
#
sub pending_actions {
  my $jobq = shift;
  my %seen;
  my $i = 0;
  for ( job_actions( $jobq ) ) {
    my( $name, $status, $time ) = @$_;
    if ( $status eq 'Requested' ) {
      $seen{$name} = $i;
    }
    elsif ( $status eq 'Performed' ) {
      delete $seen{$name};
    }

    $i++;
  }
  sort { $seen{$a} <=> $seen{$b} } keys %seen;
}

#
# look up the latest status string for the named action:
#
sub action_status {
  my $jobq = shift;
  my $action_name = shift;
  for ( reverse $jobq->param('action' ) ) {
    my( $name, $status, $time ) = split / - /;
    if ( $name eq $action_name ) {
      return $status;
    }
  }
  undef;
}


package Id;


my @words;
my %words;
my $id_space = 8_000_000;

BEGIN {
    if ( open WF, "< words.6" ) {
      @words = <WF>;
      close WF;
      chomp @words;
      @words{ @words } = ( 0 .. $#words );
      $id_space = @words ** 2;
    }
    else {
      print STDERR "error opening words.6: $!";
      return 0;
    }
}

sub validate_name {
  my $name = shift;
  if ( $name =~ /^([a-z]{6})[^a-z]([a-z]{6})$/ ) {
    if ( exists $words{$1} && exists $words{$2} ) {
      return "$1-$2"; # canonical form.
    }
  }
  0; # not valid
}

sub new {
  my $p = shift;
  my $id = int rand( $id_space );
  bless {
    id => $id,
    mnem => id_to_mnemonic( $id ),
  }, $p;
}

sub old {
  my $p = shift;
  my $val = shift;
  my $mnem;
  my $id;
  if ( $val =~ /^([a-z]{6})[^a-z]([a-z]{6})$/ ) {
    if ( exists $words{$1} && exists $words{$2} ) {
      return bless {
        mnem => $val,
        id => mnemonic_to_id( $1, $2 ),
      }, $p;
    }
  }
  elsif ( $val =~ /^\d+$/ ) {
    return bless {
      id => $val,
      mnem => id_to_mnemonic( $val ),
    }, $p;
  }
  die "Id::old( $val ) -- Bogus value!\n";
}

sub numeric {
  my $self = shift;
  $self->{'id'};
}

sub mnemonic {
  my $self = shift;
  $self->{'mnem'};
}

sub id_to_mnemonic {
  my $id = shift;
  my $upper = int( $id / @words );
  my $lower = int( $id % @words );
  $words[ $upper ] . '-' . $words[ $lower ];
}

sub mnemonic_to_id {
  my $mnem = shift;
  my( $l, $r ) = @_;
  my %h; @h{ @words } = ( 0 .. $#words );
  $h{$l} * @words + $h{$r};
}

1;

