#!/usr/local/bin/perl -w

# this expects one or more of the following CGI parameters:
#   job_name
#   keywords
#   new_action
#   uploaded_file

use CGI;
use CGI::Carp qw(fatalsToBrowser);

use lib ('.');
use JobCommon;

print my_header;

# print "<b>CGI Query:</b><br>", $query->dump;

my $job_name = $query->param( 'job_name' ) || $query->param( 'keywords' );

my $jobq;

if ( $job_name ) {
  $jobq = load_job( $job_name );

  unless ( $jobq ) {
    # error loading job
    die "Error: unable to load job '$job_name' (no such job?)\n";
  }

  if ( defined $query->param( 'keywords' ) ) {
    print $query->b("Bookmark this page now, to check this job in the future.");
  }

  if ( defined $query->param( 'new_action' ) ) {
    add_action( $jobq, $query->param('new_action') );
  }
}

if ( defined $query->param('uploaded_file') ) {
  $jobq = enqueue_job( $query );
}

if ( $jobq ) {
  check_job_status( $jobq );

  print 
    $query->b( "Request another Action to be performed on this Collection:" ),
    $query->startform( 'POST', $script_name ),
    ( map {
        $query->submit( 'new_action', $_ ), $query->br
      } qw( Null-Short Null-Long ) ),
    $query->hidden( 'job_name', $jobq->param('job_name') ),
    $query->endform;
}

# print "<b>Job Record:</b><br>", $jobq->dump;

print 
  $query->b(
    $query->isindex(
      undef, # name of 'action' script. undef means come back to this one.
      q{PROMPT="Enter the name of a job to check (e.g. 'sample-sample'): "}
    )
  ),
  $query->i("This page last loaded at ",scalar localtime),
  $query->end_html;

exit 0;


sub check_job_status {
  my $jobq = shift;

  my $job_name = $jobq->param('job_name');

  print 
    $query->h3( qq{Status of job <font color="#004488">$job_name</font>:\n} ),
    $query->ul( map { 
      $query->li( "$_->[0] - $_->[1] - ".scalar(localtime $_->[2]) )
    } job_actions( $jobq ) ),
    $query->p;

  #
  # check to see if anything is ready for downloading,
  # but only if no actions remain unperformed.
  #
  unless ( pending_actions( $jobq ) ) {
    my @files = glob( "$outgoing_dir/$job_name.*" );
    if ( @files ) {
      print 
        "You may now download the results of the the annotation:\n",
        $query->br,
        $query->ul( map {
          s,$outgoing_dir/,,;
          $query->li( $query->a({-HREF => "$outgoing_dir/$_"}, $_ ) );
        } @files );
    }
  }

  $jobq;
}





#
# adds params 'collection_file' and 'incoming_time'
# the the cgi query struct.
#
sub enqueue_job {
  my $jobq = shift; # the original CGI query object

  my $filename    = $jobq->param('uploaded_file');
  my $tmpfilename = $jobq->tmpFileName( $filename );

  close $filename;

  my $ext;

  {
    local $_ = $filename;
    if ( /\.tar$/i
      or /\.tar\.z$/i
      or /\.tar\.gz$/i
      or /\.z$/i
      or /\.gz$/i
      or /\.trz$/i
      or /\.tgz$/i
    ) {
      $ext = $&;
    }
  }

  unless ( $ext ) {
    print "Not a valid archive format!\n";
    return undef;
  }

  my $nId = new Id;
  my $job_name = $nId->mnemonic;
  my $newfilename = $job_name . $ext;

  $jobq->param( -NAME => 'job_name', -VALUE => $job_name );
  $jobq->param( -NAME => 'collection_file', -VALUE => $newfilename );

  add_action_item( $jobq, 'Upload', 'Performed' );

  system("cp $tmpfilename $incoming_dir/$newfilename");

  save_job( $jobq );

  $jobq;
}



sub add_action {
  my( $jobq, $action_name ) = @_;

  add_action_item( $jobq, $action_name, 'Requested' );

  #
  # start a fulfiller for this guy if there isn't one already:
  #
  unless ( defined $jobq->param('fulfiller') ) {
    my $f = "$jobs_dir/$job_name.fulfiller";
    if ( open F, "> $f" ) {
      print F "fulfill_job $job_name";
      close F;
      for ( `at -f $f -s now + 1 minutes 2>&1` ) {
        if ( /^job (\S+) / ) {
          $jobq->param( -NAME => 'fulfiller', -VALUE => $1 );
        }
      }
    }
  }

  save_job( $jobq );

  $jobq;
}


